@@ -55,12 +55,12 @@
 //! It may seem reasonable to expect numbers that differ by less than 0.000001 to be equal, but
 //! this does not always work well (consider comparing -0.0000000028 to +0.00000097).
 
-extern crate num_traits as num;
+extern crate num;
 use num::Zero;
 
 use std::mem;
 use std::cmp::{Ordering,PartialOrd};
-use std::ops::{Sub,Div};
+use std::ops::{Sub,Div,Neg};
 
 /// A trait for floating point numbers which computes the number of representable
 /// values or ULPs (Units of Least Precision) that separate the two given values.
@@ -427,22 +427,40 @@ fn f64_approx_cmp_test2() {
 
 /// ApproxEqRatio is a trait for approximate equality comparisons bounding the ratio
 /// of the difference to the larger.
-pub trait ApproxEqRatio : Div<Output = Self> + Sub<Output = Self> + PartialOrd + Zero
-    + Sized + Copy
+pub trait ApproxEqRatio : Div<Output = Self> + Sub<Output = Self> + Neg<Output = Self>
+    + PartialOrd + Zero + Sized + Copy
 {
     /// This method tests if `self` and `other` are nearly equal by bounding the
     /// difference between them to some number much less than the larger of the two.
     /// This bound is set as the ratio of the difference to the larger.
     fn approx_eq_ratio(&self, other: &Self, ratio: Self) -> bool {
+
+        // Not equal if signs are not equal
         if *self < Self::zero() && *other > Self::zero() { return false; }
         if *self > Self::zero() && *other < Self::zero() { return false; }
-        let (smaller,larger) = if *self < *other {
-            (self,other)
+
+        // Handle all zero cases
+        match (*self == Self::zero(), *other == Self::zero()) {
+            (true,true) => return true,
+            (true,false) => return false,
+            (false,true) => return false,
+            _ => { }
+        }
+
+        // abs
+        let (s,o) = if *self < Self::zero() {
+            (-*self, -*other)
         } else {
-            (other,self)
+            (*self, *other)
         };
-        let difference: Self = larger.sub(*smaller);
-        let actual_ratio: Self = difference.div(*larger);
+
+        let (smaller,larger) = if s < o {
+            (s,o)
+        } else {
+            (o,s)
+        };
+        let difference: Self = larger.sub(smaller);
+        let actual_ratio: Self = difference.div(larger);
         actual_ratio < ratio
     }
 
@@ -475,6 +493,34 @@ fn f32_approx_eq_ratio_test2() {
     assert!(y.approx_ne_ratio(&x, 0.79));
 }
 
+#[test]
+fn f32_approx_eq_ratio_test_zero_eq_zero_returns_true() {
+    let x: f32 = 0.0_f32;
+    assert!(x.approx_eq_ratio(&x,0.1) == true);
+}
+
+#[test]
+fn f32_approx_eq_ratio_test_zero_ne_zero_returns_false() {
+    let x: f32 = 0.0_f32;
+    assert!(x.approx_ne_ratio(&x,0.1) == false);
+}
+
+#[test]
+fn f32_approx_eq_ratio_test_against_a_zero_is_false() {
+    let x: f32 = 0.0_f32;
+    let y: f32 = 0.1_f32;
+    assert!(x.approx_eq_ratio(&y,0.1) == false);
+    assert!(y.approx_eq_ratio(&x,0.1) == false);
+}
+
+#[test]
+fn f32_approx_eq_ratio_test_negative_numbers() {
+    let x: f32 = -3.0_f32;
+    let y: f32 = -4.0_f32;
+    // -3 and -4 should not be equal at a ratio of 0.1
+    assert!(x.approx_eq_ratio(&y,0.1) == false);
+}
+
 impl ApproxEqRatio for f64 { }
 
 #[test]
@@ -494,3 +540,24 @@ fn f64_approx_eq_ratio_test2() {
     assert!(x.approx_eq_ratio(&y, 0.81));
     assert!(y.approx_ne_ratio(&x, 0.79));
 }
+
+#[test]
+fn f64_approx_eq_ratio_test_zero_eq_zero_returns_true() {
+    let x: f64 = 0.0_f64;
+    assert!(x.approx_eq_ratio(&x,0.1) == true);
+}
+
+#[test]
+fn f64_approx_eq_ratio_test_zero_ne_zero_returns_false() {
+    let x: f64 = 0.0_f64;
+    assert!(x.approx_ne_ratio(&x,0.1) == false);
+}
+
+#[test]
+fn f64_approx_eq_ratio_test_negative_numbers() {
+    let x: f64 = -3.0_f64;
+    let y: f64 = -4.0_f64;
+    // -3 and -4 should not be equal at a ratio of 0.1
+    assert!(x.approx_eq_ratio(&y,0.1) == false);
+}
+
